@@ -1,8 +1,11 @@
+
 import certifi
 from os import path, environ
 from flask import Flask
 
-from eisenradio.instance.config_apfac import write_config, remove_config
+from eisenradio.instance.config_apfac import write_config_test, remove_config
+import eisenradio.instance.config as config
+
 from eisenradio.api import api, ghettoApi
 
 # android ssl fix
@@ -15,42 +18,35 @@ script_path = path.dirname(__file__)
 """ 'Home' and 'Util' load modules, templates, styles, favicon from its own project folders """
 
 
-def create_app(work_port, test=None):  # must be None
-    """
+def create_app(work_port):  # must be None
+    """prod
+
     prod
     """
-
-    app = Flask(__name__,
-                instance_path=path.join(script_path, 'instance'))
+    app = Flask('eisenradio')
 
     with app.app_context():
 
-        app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0
         api.init_app(app)
         ghettoApi.init_work_port(work_port)    # port to use; browser autostart, sound endpoint
 
-        if test is not None:
-            """------------- TEST RUN -------------------
-            exec  write_config('test') at first from pytest"""
-            write_config('test')
-            # remove in teardown
+        is_snap_device = 'SNAP' in environ  # write in [SNAP_USER_COMMON]
 
-        if not test:
-            # environ['WERKZEUG_RUN_MAIN'] = 'true'
-            is_snap_device = 'SNAP' in environ  # write in [SNAP_USER_COMMON]
-            is_android_device = 'ANDROID_STORAGE' in environ
+        is_android_device = 'ANDROID_STORAGE' in environ
 
-            if not is_snap_device and not is_android_device:
-                write_config('prod')
-                remove_config()
+        if not is_snap_device and not is_android_device:
+            # PROD
+            app.config.from_object('config.ProdConfig')
+            # app.config.from_object('config.TestConfig')    # total fail from pytest
 
-            if is_snap_device:
-                write_config('snap')
-                remove_config(environ["SNAP_USER_COMMON"])
+        if is_snap_device:
+            # write_config('snap')
+            # remove_config(environ["SNAP_USER_COMMON"])
+            app.config.from_object('config.SnapConfig')
+            pass
 
-            if is_android_device:
-                write_config('android')
-                # db must exist (write problem with schema not solved so far), os and flask env from module
+        if is_android_device:
+            app.config.from_object('config.AndroidConfig')
 
         # helper stuff
         from eisenradio.lib.platform_helper import main as start_frontend
@@ -65,15 +61,13 @@ def create_app(work_port, test=None):  # must be None
 
         create_install_db(app.config['DATABASE'])
 
-        if test:
-            print(f"\tapp.config:       {app.config}\n")
-        if not test:
-            start_frontend()
+        start_frontend()
         return app
 
 
 def create_app_dev(work_port, test=None):  # must be None
-    """
+    """dev
+
     dev
     """
     app = Flask(__name__,
@@ -88,7 +82,7 @@ def create_app_dev(work_port, test=None):  # must be None
         if test is not None:
             """------------- TEST RUN -------------------
             exec  write_config('test') at first from pytest"""
-            write_config('test')
+            # write_config('test')
             # remove in teardown
 
         if not test:
@@ -97,8 +91,9 @@ def create_app_dev(work_port, test=None):  # must be None
             is_android_device = 'ANDROID_STORAGE' in environ
 
             if not is_snap_device and not is_android_device:
-                write_config('dev')
-                remove_config()
+                # write_config('dev')
+                # remove_config()
+                app.config.from_object('config.DevConfig')
 
         # helper stuff
         from eisenradio.lib.platform_helper import main as start_frontend
@@ -116,6 +111,5 @@ def create_app_dev(work_port, test=None):  # must be None
         if test:
             print(f"\tTEST!!!:       {app.config}\n")
         if not test:
-            print(f"\tDEV!!!:       {app.config}\n")
             start_frontend()
         return app
