@@ -2,12 +2,12 @@ import sys
 import random
 from os import path
 
-this_dir = path.abspath(path.join(path.dirname("__file__")))
-api_dir = path.abspath(path.join(path.dirname("__file__"),   'api'))
-home_dir = path.abspath(path.join(path.dirname("__file__"),  'eisenhome'))
-utils_dir = path.abspath(path.join(path.dirname("__file__"), 'eisenutils'))
-instance_dir = path.abspath(path.join(path.dirname("__file__"), 'instance'))
-lib_dir = path.abspath(path.join(path.dirname("__file__"),   'lib'))
+this_dir = path.abspath(path.join(path.dirname(__file__)))
+api_dir = path.abspath(path.join(path.dirname(__file__),   'api'))
+home_dir = path.abspath(path.join(path.dirname(__file__),  'eisenhome'))
+utils_dir = path.abspath(path.join(path.dirname(__file__), 'eisenutils'))
+instance_dir = path.abspath(path.join(path.dirname(__file__), 'instance'))
+lib_dir = path.abspath(path.join(path.dirname(__file__),   'lib'))
 sys.path.append(path.abspath(this_dir))
 sys.path.append(path.abspath(api_dir))
 sys.path.append(path.abspath(home_dir))
