@@ -4,28 +4,29 @@ from eisenradio.api import ghettoApi
 
 
 def stream_audio_feed(name):
-
+    """yield buffer, drain queue (cleanup) in ghetto_audio_stream_dict[name + ',audio'] if listen btn gets inactive"""
     while True:
 
         try:
-            # drain the audio element as fast as possible for switching stations
             if not ghettoApi.listen_active_dict[name]:
+                if not ghettoApi.ghetto_audio_stream_dict[name + ',audio'].empty():
+                    ghettoApi.ghetto_audio_stream_dict[name + ',audio'].get()
                 return
-            # grab from queue
+
             if not ghettoApi.ghetto_audio_stream_dict[name + ',audio'].empty():
-                chunk = ghettoApi.ghetto_audio_stream_dict[name + ',audio'].get()
-                yield chunk
+                yield ghettoApi.ghetto_audio_stream_dict[name + ',audio'].get()
+
         except KeyError:
             pass
 
 
 def get_stream_content_type(name):
+    """loop until return radios content type, create var {radio: content type}, route don't call def again"""
     content_type = ''
     while not content_type:
 
         try:
             content_type = ghettoApi.ghetto_measure_dict[name + ',suffix']
-            """os variable, can resp. faster next time"""
             environ[name] = content_type
         except KeyError:
             pass
