@@ -1,36 +1,49 @@
+"""Tools menu Tools function except Monitor Records (monitor_records.py)
+
+delete all radios - from db
+    delete_all_radios()
+export radios - names and urls
+    export_radios()
+export blacklists
+    dump_radio_blacklist()
+import radios - upload, since we are a webserver
+    upload_radios() - master function, upload_ prefix
+    radio_spare_image() - get random image for radio
+import blacklists
+    upload_blacklists() - master function, upload_ prefix
+"""
 import configparser
 import json
 import shutil
 import random
-import threading
 
 import eisenradio.lib.eisdb as eisen_db
 import eisenradio.eisenutil.monitor_records as mo_rec
 
 from os import path, remove, listdir
-from flask import flash
 from eisenradio.eisenutil.eisutil import is_in_db_view
 
 
 def delete_all_radios():
-    no_error = True
+    """delete all radios from db, return True if db is empty"""
     posts = get_radios()
-
     for radio in posts:
-        rv = eisen_db.delete_radio(radio['id'])
-        if not rv:
-            no_error = False
-    # vakuum db
-    eisen_db.get_db_smaller()
-    if no_error:
-        flash('Delete all radios. Done.', 'success')
-    else:
-        flash('not all radios where deleted', 'warning')
-    return no_error
+        eisen_db.delete_radio(radio['id'])
+
+    posts = get_radios()
+    if not len([radio for radio in posts]):
+        return True
+    return False
 
 
 def export_radios():
-    # BLUES_UK = http://149.255.59.3:8232/stream
+    """write dump of radio names and urls to ini file and return True
+
+    return False if no radios in db or a physical download directory is non-existing
+    make backup of existing ini file
+    write download path to [GLOBAL] Section for easy restore
+    write radio names and urls to [SECTION] (BLUES_UK = http://149.255.59.3:8232/stream)
+    """
     posts = get_radios()
     if not posts:
         return False
@@ -56,6 +69,7 @@ def export_radios():
 
 
 def export_radios_dump(posts):
+    """return radio_url_dict {radio name: url} and download_path"""
     radio_url_dict = {}
     download_path = ''
     for station in posts:
@@ -67,20 +81,28 @@ def export_radios_dump(posts):
 
 
 def upload_radios(str_ini):
+    """call upload_radios_db_import_ini() to upload radios and urls from ini file
+
+    return True, return False if no [STATIONS] section for config parser in file
     """
-    Master def of uploading radios and urls
-    """
+
     config = configparser.ConfigParser()
     config.optionxform = str  # not squeeze the names lower case
     download_path = upload_radios_read_global(config, str_ini)
     name_url_dict = upload_radios_read_stations(config, str_ini)
     if not name_url_dict:
-        return False
-    upload_radios_db_import_ini(name_url_dict, download_path)
-    return True
+        return False, None
+    num_added_db = upload_radios_db_import_ini(name_url_dict, download_path)
+    return True, num_added_db
 
 
 def upload_radios_read_global(config, str_ini):
+    """read [GLOBAL] section into a dict, if there is one, to return the download path, return Nothing if none
+
+    some ini don't own a [GLOBAL] section
+    GhettoRecorder has no download path in [GLOBAL], ignore error for such files
+    """
+
     try:
         config.read_string(str_ini)
         global_dict = config['GLOBAL']
@@ -97,6 +119,7 @@ def upload_radios_read_global(config, str_ini):
 
 
 def upload_radios_read_stations(config, str_ini):
+    """return config parser [STATIONS] as dict or return Nothing"""
     try:
         config.read_string(str_ini)
         station_dict = config['STATIONS']
@@ -108,11 +131,15 @@ def upload_radios_read_stations(config, str_ini):
 
 
 def upload_radios_db_import_ini(station_dict, download_ini_path=None):
+    """import names and urls from station_dict into db, function returns nothing
+
+    keyword argument: download_ini_path - two scenarios for None
+        restore all radios after complete deletion,
+        append new radios and no download path available
+    import only unknown radios
+    download path is preferred from db, then ini, then write a message as path
     """
-    imports only new radios with url from ini file into db, [STATION] section, download_path is in [GLOBAL]
-    two scenarios, restore all after complete deletion, append new radios
-    download path is preferred from db, then ini, then put a message as path
-    """
+
     conn = eisen_db.get_db_connection()
     posts = conn.execute('SELECT * FROM posts').fetchall()
     download_db_path = upload_radios_db_get_download_path(posts)
@@ -124,6 +151,7 @@ def upload_radios_db_import_ini(station_dict, download_ini_path=None):
     else:
         download_path = '/Please/use/Save/from/menu'
 
+    counter = 0
     for radio, url in station_dict.items():
         if is_in_db_view(radio):
             continue
@@ -131,11 +159,13 @@ def upload_radios_db_import_ini(station_dict, download_ini_path=None):
         conn.execute('INSERT INTO posts (title, content, download_path, pic_data, pic_content_type) VALUES ('
                      '?, ?, ?, ?, ?)', (radio, url, download_path, radio_image, image_content_type))
         conn.commit()
-
+        counter += 1
     conn.close()
+    return counter
 
 
 def upload_radios_db_get_download_path(posts):
+    """return download path for app, except there is no return Nothing"""
     try:
         download_path = posts[0]["download_path"]
     except Exception as error:
@@ -146,6 +176,7 @@ def upload_radios_db_get_download_path(posts):
 
 
 def get_radios():
+    """return a db dump"""
     conn = eisen_db.get_db_connection()
     posts = conn.execute('SELECT * FROM posts').fetchall()
     conn.close()
@@ -153,9 +184,7 @@ def get_radios():
 
 
 def radio_spare_image():
-    """
-    get a random image from static folder
-    """
+    """return a random image with content_type from own package folder"""
     this_dir = path.dirname(__file__)
     # rip off one dir
     app_root = path.dirname(this_dir)
@@ -169,6 +198,7 @@ def radio_spare_image():
 
 
 def get_export_path():
+    """return the download parent path for radios, or a helper string if no path exists yet"""
     download_path = eisen_db.get_download_dir()
     if not download_path:
         export_path = "use Save from Navigation Menu"
@@ -178,11 +208,13 @@ def get_export_path():
 
 
 def dump_radio_blacklist():
-    """
-    create a dump_dict[radio] = blacklist dictionary for all radios with a list to export it to a file
-    parent folder for radios is used as place to store the file (json dict)
-    first dict entry is a message
-    import only if Key (RadioName) is matching, upper, lower case
+    """call dump_write_radio_blacklist() to dump db radio blacklists to one file in json format, return True
+
+    return False if export_path (download_path) not in db
+    return False if function call fails, or True if ok
+    use parent folder of radios for file storage (json dict)
+    create a dump_dict[radio], blacklist dictionary for all radios with a list in value field to export it to a file
+    write a message to dump_dict as first entry to clarify file content
     """
     dump_dict = {}
     conn = eisen_db.get_db_connection()
@@ -216,6 +248,7 @@ def dump_radio_blacklist():
 
 
 def dump_write_radio_blacklist(dump_dict, export_path):
+    """dump db list to json file on disk return True if no OS error"""
     dump_file_path = path.join(export_path, 'eisenradio_blacklists.json')
     dump_file_path_backup = path.join(export_path, 'eisenradio_blacklists.json_backup')
     dump_radio_export_backup(dump_file_path_backup, dump_file_path)
@@ -230,9 +263,7 @@ def dump_write_radio_blacklist(dump_dict, export_path):
 
 
 def dump_radio_export_backup(dump_file_path_backup, dump_file_path):
-    """
-    make a copy from last backup before overwrite file
-    """
+    """make a copy from last backup before overwrite file"""
     if path.exists(dump_file_path_backup):
         remove(dump_file_path_backup)
     if path.exists(dump_file_path):
@@ -240,16 +271,19 @@ def dump_radio_export_backup(dump_file_path_backup, dump_file_path):
 
 
 def upload_blacklists(json_file):
-    """
-    called from routes.tools_upload_trace_lists(), delivers the json_file raw, must be loaded
-    dump_dict[radio] is the import list for the radio
-    returns False on error to display a warning on html
+    """return True if dict from json is not empty and no import error
+
+    convert json format to dump_dict
+    feed upload_blacklists_import() with list of all radios and dict
+    write html if ok or error
     """
     upload_ok = True
     dump_dict = upload_blacklists_dump(json_file)
-    if not len(dump_dict) > 0:
-        return False
 
+    if not len(dump_dict) > 0:
+        return False, None
+
+    title_count = len([line for f_list in dump_dict.values() if type(f_list) is list for line in f_list])
     radios_list = upload_blacklists_get_radio_names_from_db()
     if radios_list:
         imported_all_ok = upload_blacklists_import(dump_dict, radios_list)
@@ -264,10 +298,15 @@ def upload_blacklists(json_file):
         print('\n\tBlacklists updated and loaded.\n')
     else:
         print('\n\tSome error occurred. See terminal messages.\n')
-    return upload_ok
+    return upload_ok, title_count
 
 
 def upload_blacklists_import(dump_dict, radios_list):
+    """return True if all radios got their lists updated
+
+    loop for radio merge db_list with import_list and write list back to db
+    import, merge only if key (RadioName) is matching, upper, lower case
+    """
     import_no_err = True
     for row in radios_list:
         radio = row[0]
@@ -283,6 +322,7 @@ def upload_blacklists_import(dump_dict, radios_list):
 
 
 def upload_blacklists_get_radio_names_from_db():
+    """return a list of all radio names in db"""
     conn = eisen_db.get_db_connection()
     radios_list = conn.execute('SELECT title FROM posts').fetchall()
     conn.close()
@@ -290,6 +330,7 @@ def upload_blacklists_get_radio_names_from_db():
 
 
 def upload_blacklists_dump(json_file):
+    """return dict format from json_file"""
     dump_dict = {}
     try:
         dump_dict = json.loads(json_file)
@@ -300,8 +341,9 @@ def upload_blacklists_dump(json_file):
 
 
 def upload_blacklists_merge(import_list, radio):
-    """
-    concatenate lists, make a set of 'em to remove double values, convert set back to list
+    """return concatenated lists and True, except unknown error
+
+    make a set of extended db_list and import_list to remove double values, convert back to list format
     """
     rv = True
     blacklist = mo_rec.dump_radio_blacklist_from_col(radio)
@@ -319,6 +361,7 @@ def upload_blacklists_merge(import_list, radio):
 
 
 def upload_blacklists_write(merged_blacklist, radio):
+    """dump lists into db and return True, except unknown error"""
     rv = True
     try:
         conn = eisen_db.get_db_connection()
@@ -330,89 +373,3 @@ def upload_blacklists_write(merged_blacklist, radio):
         print(message)
         rv = False
     return rv
-
-
-def tool_aacp_repair(file_dict):
-    """
-    corrective header and tail of file if aacp or aac
-    """
-    [print(f'{name}') for name in file_dict.keys()]
-
-    threading.Thread(name="aacp_repair_daemon", target=tool_aacp_repair_file,
-                     args=[file_dict],
-                     daemon=True).start()
-
-
-def tool_aacp_repair_file(file_dict):
-    export_path = eisen_db.get_download_dir()
-    tail_repaired = None
-    ok_list = []
-    fail_list = []
-    for file_name, file_content in file_dict.items():
-        file_path = path.join(export_path, file_name)
-        head_repaired = tool_aacp_repair_head(file_content)
-        if head_repaired is not None:
-            tail_repaired = tool_aacp_repair_tail(head_repaired)
-        else:
-            fail_list.append(file_name + "\n")
-        if tail_repaired is not None:
-            ok_list.append(file_name + "\n")
-            with open(file_path, 'wb') as binary_writer:
-                binary_writer.write(tail_repaired)
-            print(f'write file: {file_path}')
-        else:
-            fail_list.append(file_name + "\n")
-
-    file_path = path.join(export_path, 'eisenradio_aacp_repair.txt')
-    ok_msg = f'\n\t----- {str(len(ok_list))} file(s) repaired -----\n'
-    fail_msg = f'\n\t----- {str(len(fail_list))} file(s) failed -----\n'
-    with open(file_path, 'w') as text_writer:
-        text_writer.write(fail_msg)
-        text_writer.writelines(fail_list)
-        text_writer.write(ok_msg)
-        text_writer.writelines(ok_list)
-
-
-def tool_aacp_repair_head(chunk):
-    hex_chunk = chunk.hex()
-    start, end = 0, 4
-    search_string = "fff1"
-    while 1:
-        if end > len(hex_chunk):
-            break
-        if hex_chunk[start:end] == search_string:
-            # return bytes slice from shifted start to the end of chunk
-            try:
-                return bytes.fromhex(hex_chunk[start:])
-            except ValueError:
-                return
-            except Exception as error:
-                message = f'unknown error in tool_aacp_repair_head(), {error} ignore it.'
-                print(message)
-                return
-        start += 1
-        end += 1
-    return
-
-
-def tool_aacp_repair_tail(chunk):
-    hex_chunk = chunk.hex()
-    start, end = -1, -5
-    search_string = "fff1"
-    while 1:
-        if end < -(len(hex_chunk)):
-            break
-        if hex_chunk[end:start] == search_string:
-            # return bytes before end variable
-            try:
-                return bytes.fromhex(hex_chunk[:end])
-            except ValueError:
-                # ValueError: non-hexadecimal number found in fromhex() arg at position 64805
-                return
-            except Exception as error:
-                message = f'unknown error in tool_aacp_repair_tail(), {error} ignore it.'
-                print(message)
-                return
-        start -= 1
-        end -= 1
-    return
