@@ -16,14 +16,31 @@
 #
 #   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 #   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+#   FITNESS FOR A PARTICULAR PURPOSE AND NON INFRINGEMENT. IN NO EVENT SHALL THE
 #   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 #   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 #   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 #   SOFTWARE.
 ###################################################################################
+"""
+connections:
+url is tested before the "real" connection to write error messages to a dict and html
+content type, bitrate and playlist (some url deliver only a playlist with real urls) is asked for
+for every connection (rec or listen) is a second connection opened to ask every few seconds for metadata, aka title
+asking for metadata in one stream leads to blocks of metadata in the datastream; digital noise and audio jumps
+if a listen connection is opened and recording is activated a second connection is opened
+this is four connections in sum for the radio, two streams, two metadata
+buttons:
+button status is written to dicts in Python and we should have an instance for each radio
+not only in JS (to apply css styles) but also here on the server
+url requests:
+use urllib with certify and ssl imports plus ssl.create_default_context(cafile=certifi.where())
+to avoid ssl errors on android and mac
+"""
 import io
 import os
+import certifi
+import ssl
 import queue
 import shutil
 import threading
@@ -32,19 +49,26 @@ import urllib.request
 from concurrent.futures import ThreadPoolExecutor
 from time import sleep, strftime, time
 from urllib.error import URLError, HTTPError
+from urllib.request import urlopen, Request
+from eisenradio.api import ghettoApi, ghettoTest
 
-import urllib3
-from eisenradio.api import ghettoApi
+
+# android ssl fix, mac seems to have same issue
+os.environ['SSL_CERT_FILE'] = certifi.where()
+context_ssl = ssl.create_default_context(cafile=certifi.where())
 
 # logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
 # logging.basicConfig(level=logging.INFO, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
 
-version = '2.1'
-print(f'ghettorecorder {version} (Eisenradio     module)')
+version = '2.3'
+print(f'ghettorecorder {version} (an Eisenradio module)')
+
+
 # ########## version #############
 
 
 class GBase:
+    """base and utils methods"""
     # class attribute
     dict_exit = {}  # all "single" started (this time four, each station)
     dict_error = {}
@@ -101,6 +125,7 @@ class GIni(GBase):  # remnant of command line version
 
     @staticmethod
     def parse_url_simple_url(radio_url):
+        """called by routes.py, def post(post_id): - should be not here"""
         url = radio_url  # whole url is used for connection to radio server
 
         # 'http:' is first [0], 'ip:port/xxx/yyy' second item [1] in list_url_protocol
@@ -111,14 +136,14 @@ class GIni(GBase):  # remnant of command line version
 
 
 class GNet(GBase):
-    http_pool = urllib3.PoolManager(num_pools=20)
+    """net based actions"""
 
     @staticmethod
     def load_url(url):
-        # returns status code, if server is alive conn.getcode()
+        # returns status code 200 if ok, conn.getcode()
         # use urllib, urllib3 causes response to wait "forever" and timeout is not working either
         # print(f' load_url {url}')
-        with urllib.request.urlopen(url, timeout=15) as response:
+        with urllib.request.urlopen(url, timeout=15, context=context_ssl) as response:
             return response.getcode()
 
     @staticmethod
@@ -143,8 +168,9 @@ class GNet(GBase):
 
     @staticmethod
     def stream_filetype_url(url, str_radio):
+        """if the server fails, it makes no sense to record something"""
         try:
-            with urllib.request.urlopen(url, timeout=15) as response:
+            with urllib.request.urlopen(url, timeout=15, context=context_ssl) as response:
                 headers = response.getheader('Content-Type')
         except Exception as ex:
             print(ex)
@@ -193,7 +219,7 @@ class GRecorder:
     ghettoApi.init_ghetto_skipped_in_session_dict(skipped_in_session_dict)
 
     @staticmethod
-    def ghetto_recorder_head(directory_save, stream_suffix, str_radio, str_action):
+    def ghetto_recorder_head(directory_save, stream_suffix, str_radio, str_action, str_type):
         """
         clean metadata from problematic characters for writing to file system, hopefully caught all
         create file name and path from current metadata for recorder (tail)
@@ -202,12 +228,12 @@ class GRecorder:
         if str_action == "listen":
             return
 
-        sleep(3)  # chance to get stream meta info, some radios do not send metadata or send filtered out garbage
+        sleep(3)  # chance to get stream meta info, some radios do not send metadata or send garbage
         first_record = True
 
         display_info = ""
         while GRecorder.record_active_dict[str_radio] or not GBase.dict_exit[str_radio]:
-            stream_info = GRecorder.current_song_dict[str_radio]  # unknown_title, None, icy metadata if avail
+            stream_info = GRecorder.current_song_dict[str_radio]  # unknown_title, can be garbage
 
             if not display_info == stream_info:
                 if not stream_info == "":
@@ -222,7 +248,9 @@ class GRecorder:
                     else:
                         fresh_file_path = directory_save + '//' + stripped_info + stream_suffix
                         GRecorder.path_record_dict[str_radio] = fresh_file_path  # NEW PATH ... ... ...
-
+            if ghettoTest.thread_killer == (str_radio, str_action, str_type):
+                print(f".. thread_killer .head. {ghettoTest.thread_killer}")
+                break
             if not GRecorder.record_active_dict[str_radio] or GBase.dict_exit[str_radio]:
                 stripped_info = GBase.remove_special_chars(stream_info)
                 fresh_file_path = directory_save + '//' + '_incomplete_' + stripped_info + stream_suffix
@@ -243,6 +271,7 @@ class GRecorder:
 
     @staticmethod
     def g_recorder_await_head(str_radio):
+        """recorder must wait to get file, path name"""
         while not GBase.dict_exit[str_radio]:
             try:
                 if GRecorder.start_write_command[str_radio]:
@@ -264,15 +293,17 @@ class GRecorder:
 
     @staticmethod
     def g_recorder_request(url):
-        # url was tested in Gnet
+        """"url was tested in Gnet with "urllib" and context (SSL) manager
+        before: test conn - urllib, work conn - urllib3 (timeout not functioning in urllib3 for client terminate conn)
+        if fixed: all runs on urllib
+        big problem here for android and mac, because of certify and ssl, MUST rewrite/remove "urllib3" 
+        to be consistent for https, https://docs.python.org/3/library/urllib.request.html
+        """""
         try:
-            request = GNet.http_pool.request('GET', url,
-                                             headers={'Connection': 'keep-alive'},
-                                             timeout=3000,
-                                             preload_content=False)
+            request = urllib.request.urlopen(url, timeout=3000, context=context_ssl)
             return request
-        except Exception as e:
-            print(e)
+        except Exception as error:
+            print(error)
             return False
 
     @staticmethod
@@ -317,8 +348,7 @@ class GRecorder:
 
     @staticmethod
     def g_recorder_record_trace(str_radio, full_file_path):
-        """
-        extract the cleaned file name (in head) from path string, add to recorder_new_title_dict
+        """extract the cleaned file name (in head) from path string, add to recorder_new_title_dict
         """
         head, tail = os.path.split(full_file_path)
         title = ""
@@ -359,13 +389,19 @@ class GRecorder:
 
     @staticmethod
     def g_recorder_write_queue(str_radio, chunk):
-        if not GRecorder.audio_stream_queue_dict[str_radio + ',audio'].full():
-            GRecorder.audio_stream_queue_dict[str_radio + ',audio'].put(chunk)
+        """empty Queue if full and write
+
+        queue runs full if browser leaves page and html audio element no longer pulls audio chunks
+        less digital noise on reconnect
+        """
+        if GRecorder.audio_stream_queue_dict[str_radio + ',audio'].full():
+            GRecorder.g_recorder_empty_queue(str_radio)
+        GRecorder.audio_stream_queue_dict[str_radio + ',audio'].put(chunk)
 
     @staticmethod
     def g_recorder_ask_bit_rate(url):
         try:
-            with urllib.request.urlopen(url, timeout=15) as response:
+            with urllib.request.urlopen(url, timeout=15, context=context_ssl) as response:
                 bit_rate = response.getheader('icy-br')
                 try:
                     if int(bit_rate) % 1 == 0:
@@ -380,12 +416,12 @@ class GRecorder:
                             print(f'g_recorder_ask_bit_rate use: {this_rate} for {url}')
                             return this_rate
                     except Exception as error:
-                        message = f'use fallback rate 320, error {error}'
+                        message = f'use fallback rate 128, error {error}'
                         print(message)
-                        return 320
+                        return 128
         except Exception as ex:
             print(ex)
-            return 320
+            return 128
 
     @staticmethod
     def g_recorder_calc_chunk_size(url):
@@ -402,33 +438,36 @@ class GRecorder:
         return stream_chunk_size
 
     @staticmethod
-    def g_recorder_rec(request, str_radio, ghetto_recorder, stream_chunk_size, full_file_path, suffix):
-        """
-        rec loop
+    def g_recorder_rec(request, str_radio,
+                       ghetto_recorder, stream_chunk_size,
+                       full_file_path, suffix, str_action, str_type):
+        """rec loop
+
         start with correct path
         must recognize change in metadata to work on header and tail of datastructure is cut out of the stream
         """
         with open(ghetto_recorder, 'wb') as record_file:
-            for chunk in request.stream(stream_chunk_size):
+            while 1:
 
                 if not full_file_path == GRecorder.path_record_dict[str_radio]:
                     # new file path found
-                    GRecorder.record_write_last(chunk, record_file, suffix)
+                    GRecorder.record_write_last(request.read(stream_chunk_size), record_file, suffix)
                     GRecorder.g_recorder_cache_the_file(str_radio,
                                                         full_file_path,
                                                         record_file,
                                                         ghetto_recorder,
                                                         stream_chunk_size)
                     GRecorder.g_recorder_reset_file_offset(record_file)
-                    GRecorder.record_write_first(chunk, record_file, suffix)
+                    GRecorder.record_write_first(request.read(stream_chunk_size), record_file, suffix)
                     full_file_path = GRecorder.path_record_dict[str_radio]
                 else:
                     # fill a current file, write chunk
-                    record_file.write(chunk)
-
+                    record_file.write(request.read(stream_chunk_size))
+                if ghettoTest.thread_killer == (str_radio, str_action, str_type):
+                    print(f"..test to kill thread, restart thread?, thread_killer .tail. {ghettoTest.thread_killer}")
+                    break
                 if not GRecorder.record_active_dict[str_radio] or GBase.dict_exit[str_radio]:  # timer dict_exit
                     GRecorder.g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_chunk_size)
-                    request.release_conn()
                     break
 
     @staticmethod
@@ -443,6 +482,7 @@ class GRecorder:
 
     @staticmethod
     def record_write_last(chunk, record_file, suffix):
+        """have to fix aac file end (clean cut) so title not stuck in a playlist"""
         if suffix == ".aacp" or suffix == ".aac":
             # ('write cleaned file tail')
             acp_tail = GRecorder.record_acp_fff1_sync_word_tail(chunk)
@@ -454,8 +494,8 @@ class GRecorder:
 
     @staticmethod
     def record_acp_fff1_sync_word_tail(chunk):
-        """
-        clean cut at the end, cut out the last bytes starting with ff f1 (if not, result is a defective payload),
+        """clean cut at the end, cut out the last bytes starting with ff f1 (if not, result is a defective payload),
+
         so browser do not stop with silent error and will play next file
         """
         hex_chunk = chunk.hex()
@@ -481,8 +521,8 @@ class GRecorder:
 
     @staticmethod
     def record_acp_fff1_sync_word_header(chunk):
-        """
-        cut files out of the stream on metadata change, so the end of a file is not likely the correct start for new
+        """cut files out of the stream on metadata change, so the end of a file is not likely the correct start for new
+
         search aacp (acp plus) frame start sequence to clean the file so browser do not stop with silent error
         convert byte stream to hex, search ff f1 index_of_chunk[0] to index_of_chunk[4]
         shift the search frame in hex to right, cut out from start and return as bytes
@@ -508,9 +548,16 @@ class GRecorder:
         return
 
     @staticmethod
-    def ghetto_recorder_tail(url, str_radio, path_to_save, suffix, str_action):
-
-        audio_stream_queue = queue.Queue(maxsize=10)  # maxsize for safety, normal one in use
+    def ghetto_recorder_tail(url, str_radio, path_to_save, suffix, str_action, str_type):
+        """ switched from urllib3 back to urllib, since ssl is more easy to implement and more radios
+        use https protocol,
+        urllib3 was used because of http pool feature, but it was an incorrect assumption
+        where to change:
+        ghetto_recorder_tail - record and listen
+        metadata_request(url) -
+        playlist_m3u(url, str_radio)
+        """
+        audio_stream_queue = queue.Queue(maxsize=5)  # for safety if listen and browser disconnects (no get - pull)
         stream_chunk_size = GRecorder.g_recorder_calc_chunk_size(url)
 
         GRecorder.audio_stream_queue_dict[str_radio + ',audio'] = audio_stream_queue
@@ -526,15 +573,24 @@ class GRecorder:
             if not GRecorder.g_recorder_make_dir(path_to_save, str_radio):
                 print(f'{str_radio} Recorder can not create directory')
                 return
-            GRecorder.g_recorder_rec(request, str_radio, ghetto_recorder, stream_chunk_size, full_file_path, suffix)
+            GRecorder.g_recorder_rec(request,
+                                     str_radio,
+                                     ghetto_recorder,
+                                     stream_chunk_size,
+                                     full_file_path,
+                                     suffix,
+                                     str_action,
+                                     str_type)
 
         if str_action == "listen":
-            for chunk in request.stream(stream_chunk_size):
+            while 1:
+                GRecorder.g_recorder_write_queue(str_radio, request.read(stream_chunk_size))
 
-                GRecorder.g_recorder_write_queue(str_radio, chunk)
+                # if ghettoTest.thread_killer == (str_radio, str_action, str_type):
+                #     print(f".. thread_killer .tail. {ghettoTest.thread_killer}")
+                #     break
                 if not GRecorder.listen_active_dict[str_radio] or GBase.dict_exit[str_radio]:
                     GRecorder.g_recorder_empty_queue(str_radio)
-                    request.release_conn()
                     break
 
     @staticmethod
@@ -577,15 +633,25 @@ class GRecorder:
         except Exception as error:
             message = f'metadata_icy_info(), {str_radio}: no or false metadata; {error}'
             print(message)
-            return False
+            return b"StreamTitle='GhettoRecorder module info\n" \
+                   b"radio returns no or false metadata including title and stream url\n" \
+                   b"radio service is active on url and port, since I am not crashed, check url part after port\n" \
+                   b"recording without titles if stream is active at all';StreamUrl='';\x00\x00"
 
     @staticmethod
     def metadata_request(url):
-
-        request = GNet.http_pool.request('GET', url,
-                                         headers={'Icy-MetaData': '1'},
-                                         preload_content=False)
-        return request
+        """pull the metadata by telling server {'Icy-MetaData': '1'},
+        get binary data block with content
+        urllib3: ssl certificate stuff for urllib3 in http_pool, but goal is to remove urllib3
+        timeout works only for keeping connection open, not for termination of connection if server is offline, hangs
+        # http_pool = urllib3.PoolManager(num_pools=20, cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
+        # request = GNet.http_pool.request('GET', url, headers={'Icy-MetaData': '1'}, preload_content=False)
+        # return request
+        """
+        request = Request(url)
+        request.add_header('Icy-MetaData', '1')
+        response = urlopen(request, timeout=15, context=context_ssl)
+        return response
 
     @staticmethod
     def metadata_get_display_extract(icy_info):
@@ -622,7 +688,7 @@ class GRecorder:
             return
 
     @staticmethod
-    def get_metadata_from_stream_loop(url, str_radio, str_action):
+    def get_metadata_from_stream_loop(url, str_radio, str_action, str_type):
         stop_while = False
         icy_info = ''
         while 1:
@@ -632,7 +698,7 @@ class GRecorder:
                 request_time = round((time() - start_time) * 1000)
                 GRecorder.metadata_header_info(response, str_radio, request_time)
                 icy_info = GRecorder.metadata_icy_info(response, str_radio)
-                response.release_conn()  # preload_content=False
+
             except HTTPError:
                 pass
             except URLError:
@@ -664,6 +730,10 @@ class GRecorder:
                     if GBase.dict_exit[str_radio] or not GRecorder.record_active_dict[str_radio]:
                         stop_while = True
                         break
+                if ghettoTest.thread_killer == (str_radio, str_action, str_type):
+                    print(f".. thread_killer .metadata. {ghettoTest.thread_killer}")
+                    stop_while = True
+                    break
                 sleep(1)
             if stop_while:
                 GRecorder.current_song_dict[str_radio] = ""
@@ -671,13 +741,15 @@ class GRecorder:
 
     @staticmethod
     def playlist_m3u(url, str_radio):
-        # returns the first server of the playlist (not only m3u)
+        # returns the first server of the playlist
         try:
-            read_url = GNet.http_pool.request('GET', url, preload_content=False)
+            request = Request(url)
+            response = urlopen(request, timeout=15, context=context_ssl)
+
         except Exception as ex:
             print(ex)
         else:
-            file = read_url.read().decode('utf-8')
+            file = response.read().decode('utf-8')
 
             m3u_lines = file.split("\n")
             # print(' \n    m3u_lines    ' + file)
@@ -685,8 +757,7 @@ class GRecorder:
             m3u_streams = []
             for row_url in m3u_lines:
                 if row_url[0:4].lower() == 'http'.lower():
-                    m3u_streams.append(row_url)  # not to lower :)
-                    # print(len(m3u_streams))
+                    m3u_streams.append(row_url)  # not to lower, double hit :)
 
             if len(m3u_streams) > 1:
                 print(f' {str_radio} Have more than one server in playlist_m3u. !!! Take first stream available.')
@@ -702,8 +773,16 @@ class GRecorder:
 
 
 def check_alive_playlist_container(str_radio, str_url):
-    # playlist url?
-    # https://streams.br.de/bayern1obb_2.m3u
+    """returns url if asked url was a play list, returns False if normal server
+
+    delete an error from dict to write a nev one if any
+    https://streams.br.de/bayern1obb_2.m3u
+    GNet.is_server_alive() writes in an error dict if server fails
+    called bx eishomwe.dispatch_record_is_alive
+    """
+    if str_radio in GBase.dict_error.keys():
+        del GBase.dict_error[str_radio]
+
     if str_url[-4:] == '.m3u' or str_url[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
         # take first from the list
         is_playlist_server = GRecorder.playlist_m3u(str_url, str_radio)
@@ -720,14 +799,17 @@ def check_alive_playlist_container(str_radio, str_url):
 
 
 def record_start_radio(str_radio, url, stream_suffix, dir_save, str_action):
+    str_type = "head"
     threading.Thread(name=str_radio + '_' + str_action + "_head", target=GRecorder.ghetto_recorder_head,
-                     args=(dir_save, stream_suffix, str_radio, str_action),
+                     args=(dir_save, stream_suffix, str_radio, str_action, str_type),
                      daemon=True).start()
+    str_type = "tail"
     threading.Thread(name=str_radio + '_' + str_action + "_tail", target=GRecorder.ghetto_recorder_tail,
-                     args=(url, str_radio, dir_save, stream_suffix, str_action),
+                     args=(url, str_radio, dir_save, stream_suffix, str_action, str_type),
                      daemon=True).start()
+    str_type = "meta"
     threading.Thread(name=str_radio + '_' + str_action + "_meta", target=GRecorder.get_metadata_from_stream_loop,
-                     args=(url, str_radio, str_action),
+                     args=(url, str_radio, str_action, str_type),
                      daemon=True).start()
 
 
